#!/usr/bin/env python3
from itertools import islice
from os import system

VIDE = 0
PION = 1

class Plateau:
    """
    le plateau de jeu, avec tous ses pions
    """
    voisins = [
        [1, 2, None, None, None, None],
        [3, 4, 2, 0, None, None],
        [4, 5, None, None, 0, 1],
        [6, 7, 4, 1, None, None],
        [7, 8, 5, 2, 1, 3],
        [8, 9, None, None, 2, 4],
        [10, 11, 7, 3, None, None],
        [11, 12, 8, 4, 3, 6],
        [12, 13, 9, 5, 4, 7],
        [13, 14, None, None, 5, 8],
        [None, None, 11, 6, None, None],
        [None, None, 12, 7, 6, 10],
        [None, None, 13, 8, 7, 11],
        [None, None, 14, 9, 8, 12],
        [None, None, None, None, 9, 13],
    ]
    numero_affichage = 0
    def __init__(self):
        self.cases = [PION for _ in range(15)]
        self.cases[12] = VIDE

    def tycat(self):
        """
        affichage du plateau dans terminology
        """
        nom_fichier = "/tmp/plateau-{}.svg".format(self.numero_affichage)
        self.numero_affichage += 1
        with open(nom_fichier, "w") as svg:
            print("<svg width='600' height='600'>", file=svg)
            print("<rect width='600' height='600' fill='white'/>", file=svg)
            cases = iter(self.cases)
            for ligne in range(5):
                for colonne, contenu in enumerate(islice(cases, ligne+1)):
                    affiche_pion(svg, ligne, colonne, contenu)
            print("</svg>", file=svg)
        system("clear")
        system("tycat {}".format(nom_fichier))

    def demande_coup(self):
        """
        demande quel coup jouer a l'utilisateur
        """
        try:
            print("Jouez, ^C pour arreter")
            depart = int(input("donnez un point de depart: "))
            if self.cases[depart] == VIDE:
                print("point de depart invalide")
                raise ValueError
            arrivee = int(input("donnez un point d'arrivee: "))
            if self.cases[arrivee] == PION:
                print("point d'arrivee invalide")
                raise ValueError
            for direction, milieu in enumerate(self.voisins[depart]):
                if milieu is not None:
                    apres_milieu = self.voisins[milieu][direction]
                    if apres_milieu is not None and apres_milieu == arrivee:
                        break
            else:
                print("mouvement invalide")
                raise ValueError
            if self.cases[milieu] == VIDE:
                print("mouvement invalide")
                raise ValueError
            return depart, milieu, arrivee
        except KeyboardInterrupt:
            raise
        except:
            return self.demande_coup()

    def joue_coup(self, coup):
        """
        joue le coup valide donne.
        un coup est un triplet depart, milieu, arrivee.
        modifie le plateau.
        """
        (depart, milieu, arrivee) = coup
        self.cases[depart] = VIDE
        self.cases[milieu] = VIDE
        self.cases[arrivee] = PION

    def __bool__(self):
        compte = 0
        for case in self.cases:
            if case == PION:
                compte += 1
                if compte == 2:
                    return True
        return False


def affiche_pion(fichier, ligne, colonne, contenu):
    """
    affiche le pion donne a un endroit sympa dans le fichier svg donne.
    noir pour un emplacement vide et blanc pour un pion.
    """
    y = 100 + ligne*100
    x = 300 + (colonne - ligne/2)*100
    fond, trait = ("white", "black") if contenu else ("black", "white")
    indice = ligne*(ligne+1)//2 + colonne
    print("<circle cx='{}' cy='{}' fill='{}' stroke='{}' r='20'/>".format(x, y-10, fond, "black"),
          file=fichier)
    print("<text x='{}' y='{}' fill='{}' \
text-anchor='middle' font-size='30'>{}</text>".format(x, y, trait, indice), file=fichier)


def main():
    """
    lance une partie de solitaire
    """
    plateau = Plateau()
    plateau.tycat()
    while plateau:
        coup = plateau.demande_coup()
        print("on joue de {} a {}".format(coup[0], coup[2]))
        plateau.joue_coup(coup)
        plateau.tycat()

    print("hourra")

try:
    main()
except KeyboardInterrupt:
    print()
    print("Au revoir !")
